********************************************************************************
*     This file collects routines for data aggregation in files e.g. in NetCDF
*     format for further reading in other programs or in plain text for easy
*     plotting in gnuplot
*     TODO: to be extended
********************************************************************************
            

      SUBROUTINE netCDFCreate(p_fName, p_coodX, p_coodY,
     c      p_latVals, p_lonVals, p_varVals)
************************************************************************
* Stores normal data (as in data on a regular grid with 1-dimensional
* longitudinal and latitudinal axes) in a self-describing NetCDF file
*
* NetCDF INFO: http://www.unidata.ucar.edu/software/netcdf/guidef/ 
*
* Input:
*  - p_fName (String): name of the data storage file (the NetCDF extension
*	".nc" is automatically appended!!)
*  - p_coodX (Integer): regular grid size
*  - p_coodY (Integer): regular grid size
*  - p_latVals (Real[p_coodY]): latitudes (coordinates on the y axis of the grid)
*  - p_latVals (Real[p_coodX]): longitudes (coordinates on the x axis of the grid)
*  - p_varVals (Real[p_coodX, p_coodY]): data values on the grid
************************************************************************

      IMPLICIT none
      INCLUDE 'netcdf.inc'
          
      ! longitude in x direction, lattitude in y direction
      INTEGER p_coodX, p_coodY !netscdf grid size
      REAL p_latVals(p_coodY), p_lonVals(p_coodX) !netcdf arrays
      REAL p_varVals(p_coodX, p_coodY) !netcdf array

      ! net cdf definition vars
      INTEGER netCdfId !file id
      INTEGER latDimId, lonDimId !Dimensions
      INTEGER latId, lonId, varId ! Variables
      INTEGER latDims(1), lonDims(1), varDims(2) ! Variable dimensions
      
      DOUBLE PRECISION missingVal
      PARAMETER(missingVal=0D0)
      
      INTEGER strlen !funcs
      CHARACTER p_fName*(*), fName*50      ! file name
      fName = p_fName
   
*     Routie start           
      CALL handleErr(nf_create(
     c   fNAme(1:strlen(fName))//".nc", 
     c   NF_CLOBBER, netCdfId) ) ! open file
     
      ! define dimensions
      CALL handleErr(
     c      NF_DEF_DIM (netCdfId, "longitude", p_coodX, lonDimId) ) ! define longitude dimension
     
      CALL handleErr(
     c      NF_DEF_DIM (netCdfId, "latitude", p_coodY, latDimId) ) ! define latitude dimension
      
      
      ! variable dimensions
      latDims(1) = latDimId
      lonDims(1) = lonDimId
      varDims(1) = lonDimId
      varDims(2) = latDimId
      
      ! variables
      
      CALL handleErr(
     c NF_DEF_VAR (netCdfId, 'longitude',NF_REAL, 1, lonDims, lonId))   
     
       CALL handleErr(
     c NF_DEF_VAR (netCdfId, 'latitude', NF_REAL, 1, latDims, latId))
     
      CALL handleErr(
     c NF_DEF_VAR (netCdfId, 'var', NF_REAL, 2, varDims, varId))
     
      ! attributes
      CALL handleErr(
     c  NF_PUT_ATT_TEXT (netCdfId, latId, 'units', 13,
     c  'degrees_north') ) ! lattitude units
     
      CALL handleErr(
     c  NF_PUT_ATT_TEXT (netCdfId, lonId, 'units', 12,
     c  'degrees_east') ) ! longitude units
     
      ! variable attributes 
      CALL handleErr(
     c  NF_PUT_ATT_TEXT (netCdfId, varId, 'comment', 2,
     c  'no') )
      
      CALL handleErr(
     c  NF_PUT_ATT_TEXT (netCdfId, varId, 'long_name', 23,
     c  'Surface air temperature') ) ! variable long name
      
      CALL handleErr(
     c  NF_PUT_ATT_TEXT (netCdfId, varId, 'units', 1,
     c  'C') ) ! variable unit
      
      CALL handleErr(
     c  NF_PUT_ATT_TEXT (netCdfId, varId, 'grid_name', 11,
     c  'lonlat_grid') ) ! grid name
      
      CALL handleErr(
     c  NF_PUT_ATT_TEXT (netCdfId, varId, 'grid_type', 6,
     c  'linear') ) ! grid type
      
      CALL handleErr(
     c  NF_PUT_ATT_DOUBLE (netCdfId, varId, 'missing_value', NF_DOUBLE,
     c  1, missingVal ) ) ! level description
      
      ! global attributes
      CALL handleErr(
     c  NF_PUT_ATT_TEXT (netCdfId, NF_GLOBAL, 'title', 31,
     c  'auto-generated NetCDF data file') )
      
      ! end of definition
      CALL handleErr (NF_ENDDEF(netCdfId))
      
      ! data storage
      CALL handleErr (
     c NF_PUT_var_REAL (netCdfId, latId, p_latVals) )
      
      CALL handleErr (
     c NF_PUT_var_REAL (netCdfId, lonId, p_lonVals) )     
      
      CALL handleErr (
     c NF_PUT_var_REAL (netCdfId, varId, p_varVals) )
      
      CALL handleErr(NF_CLOSE (netCdfId))  ! close: save new netCDF dataset
   
      END
